@@ -16,7 +16,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
-__version__ = "5.0.0"
+__version__ = "5.1.0"
 __description__ = "Command Line Interface for Apache CloudStack"
 __maintainer__ = "The Apache CloudStack Team"
 __maintaineremail__ = "dev@cloudstack.apache.org"
@@ -28,7 +28,7 @@ try:
     import os
     import sys
 
-    from ConfigParser import ConfigParser, SafeConfigParser
+    from ConfigParser import ConfigParser
     from os.path import expanduser
 except ImportError, e:
     print "ImportError", e
@@ -62,10 +62,13 @@ config_fields['server']['path'] = '/client/api'
 config_fields['server']['port'] = '8080'
 config_fields['server']['protocol'] = 'http'
 config_fields['server']['timeout'] = '3600'
+config_fields['server']['expires'] = '600'
 
 # user
 config_fields['user']['apikey'] = ''
 config_fields['user']['secretkey'] = ''
+config_fields['user']['username'] = ''
+config_fields['user']['password'] = ''
 
 
 def write_config(get_attr, config_file, first_time=False):
